/*

Files     	: RGB_LED_MatrixGCC.c
Version 	: 1.1
Date     	: 2012/11/09
Description	:
History		:

    1. Date			: 2012/08/15
       Author		: John Zhong@SimplyTronics
       Modification	: Create
	   

    2. Date			: 2012/11/09
       Author		: John Zhong@SimplyTronics
       Modification	: Update the API
	   

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/



#include <RGB_LED_Matrix.h>

RGB_LED_Matrix::RGB_LED_Matrix()
{

}

void RGB_LED_Matrix::start()
{
	Serial.begin(19200);
}

void RGB_LED_Matrix::tx_data_packet(byte command, long value_32bit)
{		
	Serial.write("PMTX");
	Serial.write(command);
	Serial.write((byte)(value_32bit >> 24));
	Serial.write((byte)(value_32bit >> 16));
	Serial.write((byte)(value_32bit >> 8));
	Serial.write((byte)value_32bit);	
}

void RGB_LED_Matrix::tx_color_data(byte *buffer, long writebytes)
{
	Serial.write(buffer, writebytes);
}

void RGB_LED_Matrix::tx_data_length(long data_length)
{
	tx_data_packet(RGB_LED_MATRIX_SEND_DATA, data_length);	
}

void RGB_LED_Matrix::set_window(long x_start, long y_start, long x_end, long y_end)
{
	tx_data_packet(RGB_LED_MATRIX_SET_WINDOW_BOUND_START_X, x_start);
	tx_data_packet(RGB_LED_MATRIX_SET_WINDOW_BOUND_START_Y, y_start);
	tx_data_packet(RGB_LED_MATRIX_SET_WINDOW_BOUND_END_X, x_end);
	tx_data_packet(RGB_LED_MATRIX_SET_WINDOW_BOUND_END_Y, y_end);
}

void RGB_LED_Matrix::set_pointer(long x, long y)
{
	
	tx_data_packet(RGB_LED_MATRIX_SET_POINTER_X, x);
	tx_data_packet(RGB_LED_MATRIX_SET_POINTER_Y, y);
}

void RGB_LED_Matrix::screen_inverse(long gdram_x)
{	
	tx_data_packet(RGB_LED_MATRIX_SCREEN_COLOR_INVERSE, gdram_x);
}

void RGB_LED_Matrix::screen_overlay(long gdram_x)
{	
	tx_data_packet(RGB_LED_MATRIX_SCREEN_OVERLAY, gdram_x);
}


void RGB_LED_Matrix::screen_minuslay(long gdram_x)
{	
	tx_data_packet(RGB_LED_MATRIX_SCREEN_MINUSLAY, gdram_x);
}

void RGB_LED_Matrix::screen_gray(long gdram_x)
{	
	tx_data_packet(RGB_LED_MATRIX_SCREEN_COLOR_GRAY,gdram_x);
}

void RGB_LED_Matrix::save_picture_eeporm(long pic_num)
{	
	tx_data_packet(RGB_LED_MATRIX_SAVE_PICTURE_EEPROM,pic_num);
}

void RGB_LED_Matrix::load_picture_eeporm(long pic_num)
{	
	tx_data_packet(RGB_LED_MATRIX_LOAD_PICTURE_EEPROM,pic_num);
}

void RGB_LED_Matrix::display_parallax_logo_nbg(long x, long y)
{
	set_pointer(x, y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_PARALLAX_LOGO_NBG, 0x000000);
}


void RGB_LED_Matrix::display_parallax_logo(long x, long y, long background_color)
{
	set_pointer(x, y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_PARALLAX_LOGO, background_color);
}


void RGB_LED_Matrix::display_propeller_hat_nbg(long x, long y)
{
	set_pointer(x, y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_PROPELLER_HAT_NBG, 0x000000);
}

void RGB_LED_Matrix::display_propeller_hat(long x, long y)
{
	set_pointer(x, y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_PROPELLER_HAT, 0x000000);
}

void RGB_LED_Matrix::display_customer_bmp1(long x, long y)
{
	set_pointer(x, y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_CUSTOMER_BMP1, 0x000000);
}

void RGB_LED_Matrix::display_customer_bmp2(long x, long y)
{
	set_pointer(x, y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_CUSTOMER_BMP2, 0x000000);
}

void RGB_LED_Matrix::display_bxGDRAM(long bxGDRAM)
{
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_BXGDRAM, bxGDRAM);
}

void RGB_LED_Matrix::copy_GDRAM_to(long bxGDRAM)
{
	tx_data_packet(RGB_LED_MATRIX_COPY_2_BXGDRAM, bxGDRAM);
}

void RGB_LED_Matrix::clear_screen(long ccolor)
{
	tx_data_packet(RGB_LED_MATRIX_FILL_GDRAM, ccolor);
}

void RGB_LED_Matrix::clear_ascii_nbg(void)
{
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_BXGDRAM, 0x000000);
}

void RGB_LED_Matrix::display_ascii_1632_nbg(char c, long start_x, long start_y, long foreground_color)
{
	tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
	set_pointer(start_x, start_y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_1632_NBG, c);
}


void RGB_LED_Matrix::display_ascii_1632(char c, long start_x, long start_y, long foreground_color, long background_color)
{
	set_colors(foreground_color, background_color);
	set_pointer(start_x, start_y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_1632, c);
}



void RGB_LED_Matrix::display_ascii_0816_nbg(char c, long start_x, long start_y, long foreground_color)
{
	tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
	set_pointer(start_x, start_y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0816_NBG, c);
}


void RGB_LED_Matrix::display_ascii_0816(char c, long start_x, long start_y, long foreground_color, long background_color)
{
	set_colors(foreground_color, background_color);
	set_pointer(start_x, start_y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0816, c);
}


void RGB_LED_Matrix::display_ascii_0508_nbg(char c, long start_x, long start_y, long foreground_color)
{
	tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
	set_pointer(start_x, start_y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0508_NBG, c);
}

void RGB_LED_Matrix::display_ascii_0508(char c, long start_x, long start_y, long foreground_color, long background_color)
{
	set_colors(foreground_color, background_color);
	set_pointer(start_x, start_y);
	tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0508, c);
}

void RGB_LED_Matrix::set_colors(long foreground_color, long background_color)
{
	tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
	tx_data_packet(RGB_LED_MATRIX_SET_BACKGROUND_COLOR, background_color);
}


void RGB_LED_Matrix::circle(long x, long y, long r, long circle_color)
{
	set_pointer(x, y);
	tx_data_packet(RGB_LED_MATRIX_SET_RADIUS, r);
	tx_data_packet(RGB_LED_MATRIX_DRAW_CIRCLE, circle_color);
}

void RGB_LED_Matrix::fill_circle(long x, long y, long r, long circle_color)
{
	set_pointer(x, y);
	tx_data_packet(RGB_LED_MATRIX_SET_RADIUS, r);
	tx_data_packet(RGB_LED_MATRIX_FILL_CIRCLE, circle_color);
}

void RGB_LED_Matrix::line(long x0, long y0, long x1, long y1, long line_color)
{
	set_pointer(x0, y0);	
	tx_data_packet(RGB_LED_MATRIX_SET_LINE_END_X, x1);
	tx_data_packet(RGB_LED_MATRIX_SET_LINE_END_Y, y1);
	tx_data_packet(RGB_LED_MATRIX_DRAW_LINE, line_color);
}

void RGB_LED_Matrix::dot(long x, long y, long dot_color)
{
	set_pointer(x, y);	
	tx_data_packet(RGB_LED_MATRIX_DRAW_DOT, dot_color);
}


void RGB_LED_Matrix::reset(void)
{
	tx_data_packet(RGB_LED_MATRIX_RESET, 0x00000000);
}

void RGB_LED_Matrix::update_screen(void)
{
	tx_data_packet(RGB_LED_MATRIX_UPDATE_SCREEN, 0x00000000);
}


void RGB_LED_Matrix::change_screen_size(long new_pseudo_screen_size)
{
	tx_data_packet(RGB_LED_MATRIX_CHANGE_SCREEN_SIZE, new_pseudo_screen_size);
}


void RGB_LED_Matrix::change_device_id(long new_id)
{
	tx_data_packet(RGB_LED_MATRIX_CHANGE_DEVICEID, new_id);
}

void RGB_LED_Matrix::change_baudrate(long new_baudrate)
{
	tx_data_packet(RGB_LED_MATRIX_SET_BAUDRATE, new_baudrate);
}


void RGB_LED_Matrix::get_system_configuration(byte * read_buffer)
{
	long index = 0;
	tx_data_packet(RGB_LED_MATRIX_GET_SYSTEM_CONFIGURATION, 0x00000000);
	while (Serial.available() > 0)
    {
        if(index < 16) // One less than the size of the array
        {
            *read_buffer = Serial.read(); // Read a character
            read_buffer++;
            index++;
		}
    }
}

void RGB_LED_Matrix::get_device_id(byte * read_buffer)
{
	long index = 0;
	tx_data_packet(RGB_LED_MATRIX_GET_DEVICEID, 0x00000000);
	while (Serial.available() > 0)
    {
        if(index < 16) // One less than the size of the array
        {
            *read_buffer = Serial.read(); // Read a character
            read_buffer++;
            index++;
		}
    }
}

void RGB_LED_Matrix::get_screen_size(byte * read_buffer)
{
	long index = 0;
	tx_data_packet(RGB_LED_MATRIX_GET_SCREEN_SIZE, 0x00000000);
	while (Serial.available() > 0)
    {
        if(index < 16) // One less than the size of the array
        {
            *read_buffer = Serial.read(); // Read a character
            read_buffer++;
            index++;
		}
    }
}