/*

Files     	: RGB_LED_MatrixGCC.c
Version 	: 1.0
Date     	: 2012/08/15
Description	:
History		:

    1. Date			: 2012/08/15
       Author		: John Zhong@SimplyTronics
       Modification	: Create
	   

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef RGB_LED_MATRIX_H
#define RGB_LED_MATRIX_H

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#define RGB_LED_MATRIX_MODE_NORMAL  (0)
#define RGB_LED_MATRIX_MODE_INVERSE      (1)
#define RGB_LED_MATRIX_MODE_OVERLAY     (2)
#define RGB_LED_MATRIX_MODE_MINUSLAY   (3)
#define RGB_LED_MATRIX_MODE_GRAY    (4)
#define RGB_LED_MATRIX_MODES ( RGB_LED_MATRIX_MODE_GRAY + 1) // How many Modes the module can support 




#define    RGB_LED_MATRIX_UPDATE_SCREEN                   (0x00)

#define    RGB_LED_MATRIX_SET_POINTER_X                   (0x03)
#define    RGB_LED_MATRIX_SET_POINTER_Y                   (0x04)
#define    RGB_LED_MATRIX_SEND_DATA                      (0x05)
#define    RGB_LED_MATRIX_SET_WINDOW_BOUND_START_X         (0x06)           // NOTICE: AFTER SET THE WINDOWS BOUND, THE POINTERX AND POINTERY
#define    RGB_LED_MATRIX_SET_WINDOW_BOUND_START_Y         (0x07)           //         WILL POINT TO WINDOWS BOUND START X AND START Y AUTOMATIC
#define    RGB_LED_MATRIX_SET_WINDOW_BOUND_END_X           (0x08)           //
#define    RGB_LED_MATRIX_SET_WINDOW_BOUND_END_Y           (0x09)          //
#define    RGB_LED_MATRIX_COPY_2_BXGDRAM                  (0x0A)
#define    RGB_LED_MATRIX_DISPLAY_BXGDRAM                (0x0B)
#define    RGB_LED_MATRIX_FILL_GDRAM                     (0x0C)         // FILL THE GDRAM WITH DATA(USER DEFINED) TXCMD(#define    RGB_LED_MATRIX_FILLGDRAM, (0x01)
/*
#define    RGB_LED_MATRIX_ROLLL_EFT                      (0x0D)
#define    RGB_LED_MATRIX_ROLL_RIGHT                     (0x0E)
#define    RGB_LED_MATRIX_ROLL_UP                        (0x0F)
#define    RGB_LED_MATRIX_ROLL_DOWN                      (0x10)
*/

#define    RGB_LED_MATRIX_SET_FOREGROUND_COLOR            (0x11)
#define    RGB_LED_MATRIX_SET_BACKGROUND_COLOR            (0x12)
#define    RGB_LED_MATRIX_DISPLAY_CHAR_0508               (0x13)
#define    RGB_LED_MATRIX_DISPLAY_CHAR_0816               (0x14)

#define    RGB_LED_MATRIX_DISPLAY_CHAR_0508_NBG            (0x15)             // NBG STAND FOR NO BACKGROUND
#define    RGB_LED_MATRIX_DISPLAY_CHAR_0816_NBG            (0x16)              // IF YOU WANT TO CLEAR THESE CHARACTER IN THE MATRIX PLEASE USE THE COMMAND #define    RGB_LED_MATRIX_DISPLAYBXGDRAM(  ) ABOVE
#define    RGB_LED_MATRIX_DISPLAY_PROPELLER_HAT           (0x17)
#define    RGB_LED_MATRIX_DISPLAY_PROPELLER_HAT_NBG        (0x18)
#define    RGB_LED_MATRIX_DISPLAY_PARALLAX_LOGO           (0x19)
#define    RGB_LED_MATRIX_DISPLAY_PARALLAX_LOGO_NBG        (0x1A)
#define    RGB_LED_MATRIX_DISPLAY_CUSTOMER_BMP1           (0x1B)
#define    RGB_LED_MATRIX_DISPLAY_CUSTOMER_BMP2           (0x1C)
#define    RGB_LED_MATRIX_DRAW_DOT                       (0x1D)

#define    RGB_LED_MATRIX_SET_LINE_END_X                   (0x1E)
#define    RGB_LED_MATRIX_SET_LINE_END_Y                   (0x1F)
#define    RGB_LED_MATRIX_DRAW_LINE                      (0x20)

#define    RGB_LED_MATRIX_SET_RADIUS                     (0x21)
#define    RGB_LED_MATRIX_DRAW_CIRCLE                    (0x22)

#define    RGB_LED_MATRIX_SAVE_PICTURE_EEPROM           (0x23)
#define    RGB_LED_MATRIX_LOAD_PICTURE_EEPROM           (0x24)
#define    RGB_LED_MATRIX_FILL_CIRCLE                    (0x25)


#define    RGB_LED_MATRIX_SCREEN_COLOR_INVERSE            (0x30)
#define    RGB_LED_MATRIX_SCREEN_OVERLAY                 (0x31)
#define    RGB_LED_MATRIX_SCREEN_MINUSLAY                (0x32)
#define    RGB_LED_MATRIX_SCREEN_COLOR_GRAY               (0x33)

#define    RGB_LED_MATRIX_DISPLAY_CHAR_1632               (0x40)
#define    RGB_LED_MATRIX_DISPLAY_CHAR_1632_NBG           (0x41)

#define    RGB_LED_MATRIX_RESET                         (0xA0)
#define    RGB_LED_MATRIX_SET_BAUDRATE                   (0xA1)
#define    RGB_LED_MATRIX_SET_MODE                       (0xA2)
#define    RGB_LED_MATRIX_GET_VERSION                 (0xA6)
#define    RGB_LED_MATRIX_GET_SCREEN_SIZE                 (0xA7)
#define    RGB_LED_MATRIX_CHANGE_SCREEN_SIZE              (0xA8)
#define    RGB_LED_MATRIX_GET_DEVICEID                   (0xA9)
#define    RGB_LED_MATRIX_CHANGE_DEVICEID                (0xAA)
#define    RGB_LED_MATRIX_GET_SYSTEM_CONFIGURATION        (0xAB)



class RGB_LED_Matrix
{
	private:
	public:
		RGB_LED_Matrix(void);
		
		void start(void);	
	
		void  tx_data_packet(byte command, long value_32bit);
		
		void  tx_color_data(byte *buffer, long writebytes);

		void  tx_data_length(long data_length);

		void  set_window(long x_start, long y_start, long x_end, long y_end);

		void  set_pointer(long x, long y);

		void  screen_inverse(long gdram_x);
		
		void  screen_overlay(long gdram_x);
		
		void  screen_minuslay(long gdram_x);
		
		void  screen_gray(long gdram_x);

		void  save_picture_eeporm(long pic_num);

		void  load_picture_eeporm(long pic_num);

		void  display_parallax_logo_nbg(long x, long y);

		void  display_parallax_logo(long x, long y, long background_color);

		void  display_propeller_hat_nbg(long x, long y);

		void  display_propeller_hat(long x, long y);

		void  display_customer_bmp1(long x, long y);

		void  display_customer_bmp2(long x, long y);

		void  display_bxGDRAM(long bxGDRAM);

		void  copy_GDRAM_to(long bxGDRAM);

		void  clear_screen(long ccolor);

		void  clear_ascii_nbg(void);
		
		void  display_ascii_1632_nbg(char c, long start_x, long start_y, long foreground_color);
		
		void  display_ascii_1632(char c, long start_x, long start_y, long foreground_color, long background_color);

		void  display_ascii_0816_nbg(char c, long start_x, long start_y, long foreground_color);

		void  display_ascii_0816(char c, long start_x, long start_y, long foreground_color, long background_color);

		void  display_ascii_0508_nbg(char c, long start_x, long start_y, long foreground_color);

		void  display_ascii_0508(char c, long start_x, long start_y, long foreground_color, long background_color);

		void  set_colors(long foreground_color, long background_color);

		void  circle(long x, long y, long r, long circle_color);
		
		void  fill_circle(long x, long y, long r, long circle_color);
		
		void  line(long x0, long y0, long x1, long y1, long line_color);

		void  dot(long x, long y, long dot_color);

		void  reset(void);

		void  update_screen(void);

		void  change_screen_size(long new_pseudo_screen_size);

		void  change_device_id(long new_id);

		void  change_baudrate(long new_baudrate);

		void  get_system_configuration(byte * read_buffer);

		void  get_device_id(byte * read_buffer);

		void  get_screen_size(byte * read_buffer);
};


#endif //RGB_LED_MATRIX_H