/*

Files     	: lib_simple_test.ino
Version 	: 1.0
Date     	: 2012/11/09
Description	:
History		:

    1. Date		: 2012/11/09
       Author		: John Zhong@SimplyTronics
       Modification	: Create
	   

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <RGB_LED_Matrix.h>

/* connect the module to pin 1 Serial Port 'Tx' */
/*#define  RGB_LED_MATRIX_SIO (1) */

#define  RGB_LED_MATRIX_COLOR_BLACK (0x00000000)
#define  RGB_LED_MATRIX_COLOR_WHITE (0x00FFFFFF)
#define  RGB_LED_MATRIX_COLOR_RED (0x000000FF)
#define  RGB_LED_MATRIX_COLOR_GREEN (0x0000FF00)
#define  RGB_LED_MATRIX_COLOR_BLUE (0x00FF0000)

/* C++ object */
RGB_LED_Matrix matrix;

/* Rainbow color pattern */
long sixteen_colors_table[16] = 
{
    0x000000FF, 
    0x006500FF, 
    0x00E000FF,
    0x00D600FF,
      
    0x00E600FF,    
    0x00FF00D7, 
    0x00FF0010, 
    0x00FF3C00,
    
    0x00FFB500,    
    0x00F7FF00, 
    0x009EFF00, 
    0x0000FF11,
    
    0x0000FF76,     
    0x0000FFE2, 
    0x0000FDFF, 
    0x00005DFF,    
};

/*******************************************************
              Start and init the serial port
              Clear the screen
*******************************************************/
void setup(void)
{
  	matrix.start();
	delay(1000);
        matrix.clear_screen(RGB_LED_MATRIX_COLOR_BLACK); /* clear the screen */
        matrix.update_screen();  
}

/*******************************************************
         Function: display the 0816 ASCII string
*******************************************************/
void string_0816NBG(char * str, long start_x, long start_y, long foreground_color)
{
    int index = 0;
    matrix.tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
    matrix.tx_data_packet(RGB_LED_MATRIX_SET_POINTER_Y, start_y);
    while(*str != '\0')
    {
        matrix.tx_data_packet(RGB_LED_MATRIX_SET_POINTER_X, start_x + (index * 8));
        matrix.tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0816_NBG, *str++);
        index++;
    }
}


void loop(void)
{
/*******************************************************
              Display 'S', 'T'
*******************************************************/
	signed long index = 0;
	for(index = 15; index > 0; index--)
	{
		matrix.clear_ascii_nbg();
		matrix.display_ascii_0508_nbg('S', index, 0, 0x00FF00FF);	
		matrix.display_ascii_0508_nbg('T', index + 3, 3, 0x00FF0000);
		matrix.update_screen();	
		delay(10);	
	}           
/*******************************************************
              Display "Rainbow" Pattern 1
*******************************************************/  
      for(index = 15; index >0; index--)
      {
          matrix.line(index, 15, 15, index, sixteen_colors_table[15 - index]);
          delay(10);  
      }  
      matrix.update_screen();     
     delay(1000);  
/*******************************************************
              Display "Rainbow" Pattern 2
*******************************************************/ 
      for(index = 0; index < 16; index++)
      {
          matrix.line(0, index, index, 0, sixteen_colors_table[index]);
          delay(10);  
      }    
      matrix.update_screen(); 
     delay(1000);  

/*******************************************************
              Display "Rainbow" Pattern 3
*******************************************************/        
      for(index = 0; index < 16; index++)
      {
          matrix.line(index, 0, index, 15, sixteen_colors_table[index]);
          delay(10);  
      }
      matrix.update_screen();
      delay(1000);  


/*******************************************************
              Display "Rainbow" Pattern 4
*******************************************************/ 
      matrix.clear_screen(RGB_LED_MATRIX_COLOR_BLACK);
      for(index = 2; index < 8; index++)
      {
          matrix.circle(7, 7, index, sixteen_colors_table[index<<1]);
          delay(10); 
      } 
      matrix.update_screen();             
      delay(1000);  

/*******************************************************
              Display "Rainbow" Pattern 5
*******************************************************/ 
      matrix.copy_GDRAM_to(0);
      for(index = 2; index < 8; index++)
      {
          matrix.circle(7, 7, index, sixteen_colors_table[index]);
          matrix.update_screen();
          delay(200);  
          matrix.display_bxGDRAM(0);
          matrix.update_screen();        
      }
      delay(1000); 
/*******************************************************
              Display "SimplyTronics" String 0816
*******************************************************/ 
      for(index = 0; index > -88; index--)
      {
          string_0816NBG((char *)"SimplyTronics", index, 0, 0x00FF0000);
          matrix.update_screen();
          matrix.clear_screen(RGB_LED_MATRIX_COLOR_BLACK); /* clear the screen */          
          delay(100);
      }      
/*******************************************************
              Display default BMP file 
Notice: You need to update the firmware with your BMP files,
              before using the display customer BMP2
*******************************************************/
      matrix.display_customer_bmp1(0, 0);    
      matrix.update_screen();        
  
      delay(2000);
      
/*******************************************************
              Screen inverse
*******************************************************/
      matrix.copy_GDRAM_to(0);
      matrix.screen_inverse(0);
      matrix.update_screen();    
      delay(1000);
/*******************************************************
              Restore the screen
*******************************************************/
      matrix.display_bxGDRAM(0);
      matrix.update_screen();    
      delay(1000);  
      
/*******************************************************
          Store the screen to bxGDRAM 1 for future use
*******************************************************/
      matrix.copy_GDRAM_to(1);
      
/*******************************************************
          Display 0508 ASCII without background color
*******************************************************/
 
      for(index = '1'; index < 'J'; index++)
      {
          matrix.display_ascii_0508_nbg(index, index%8, index%8, 0x00FFFFFF);           
          matrix.update_screen();    
          delay(200);
          matrix.display_bxGDRAM(0);
      }
      
/*******************************************************
          Display 0816 ASCII without background color
*******************************************************/
      
      for(index = '1'; index < 'J'; index++)
      {
          matrix.display_ascii_0816_nbg(index, index%8, 0, 0x00FFFFFF);           
          matrix.update_screen();    
          delay(200);
          matrix.display_bxGDRAM(0);
      }   
      
/*******************************************************
          Display 0508 ASCII with background color
*******************************************************/        
      for(index = '1'; index < 'J'; index++)
      {
          matrix.display_ascii_0508(index, index%8, index%8, sixteen_colors_table[index%16], 0x000F0000);           
          matrix.update_screen();    
          delay(200);
          matrix.clear_screen(RGB_LED_MATRIX_COLOR_BLACK); 
      }
/*******************************************************
          Display 00816 ASCII with background color
*******************************************************/  
      for(index = '1'; index < 'J'; index++)
      {
          matrix.display_ascii_0816(index, index%8, 0, sixteen_colors_table[index%16], 0x000F0000);           
          matrix.update_screen();    
          delay(200);
          matrix.clear_screen(RGB_LED_MATRIX_COLOR_BLACK); 
      }

/*******************************************************
              Restore the screen
*******************************************************/
      matrix.display_bxGDRAM(1);
      matrix.update_screen();    
      delay(200);

/*******************************************************
 Display built in Parallax LOGO without background color
*******************************************************/
      matrix.set_colors(0x00FF00FF, 0x000000);
      matrix.copy_GDRAM_to(0);
      for(index = 0; index > -72; index--)
      {
          matrix.display_bxGDRAM(0);
          matrix.display_parallax_logo_nbg(index, 0);
          matrix.update_screen();  
          delay(100); 
      }
/*******************************************************
   Display built in Parallax LOGO with background color
*******************************************************/
      matrix.set_colors(0x00FFFFFF, 0x5A0000);  
      for(index = 0; index > -72; index--)
      {
          matrix.display_parallax_logo(index, 0, 0x005A0000);
          matrix.update_screen();  
          delay(100);
          matrix.clear_screen(RGB_LED_MATRIX_COLOR_BLACK);       
      }
}

